var util         = require("util");
var EventEmitter = require("events").EventEmitter;

var nats = require('nats').connect();

var config;
var child;


function HumixModule(conf) {

    var self = this;
    if (!(this instanceof HumixModule))
        return new HumixModule(config);

    config = conf;
    if(!config){

        // looking for default config
        console.log('Load local config...');
        config = require('./module.js');
        
    }

    EventEmitter.call(this);
    
    // Register Command Callback

    var prefix = 'humix.sense.'+config.module_name+".command";

    for ( var i in config.commands){

        var topic = prefix + "." + config.commands[i];

        console.log("subscribing topic:"+ topic);
        nats.subscribe(topic, function(data){

            console.log('emitting command:' + config.commands[i]);
            self.emit(config.commands[i],data);
        })
        
    }

    // register module to humix sense controller
    
    var register_config = {};
    register_config.commands = [];
    register_config.events = [];
    
    register_config.module_name = config.module_name;
    for ( var i in config.commands){
        register_config.commands.push("humix.sense."+config.module_name+".command."+config.commands[i]);
    }

    for ( var i in config.events){
        register_config.events.push("humix.sense."+config.module_name+".event."+config.events[i]);
    }
        
    
    nats.publish('humix.sense.'+config.module_name+'.status.register', JSON.stringify(register_config));

    

    // subscribe module health check status

    nats.subscribe('humix.sense.'+config.module_name+'.status.ping', function(request, replyto){

        nats.publish(replyto, 'humix.sense.'+config.module_name+'.status.pong');
    })


    console.log("config:"+JSON.stringify(config));
    // if child process is defined in the config file, start child process here
    if(config.child_process){

        var respawn = require('respawn');

        var process_option = {

            maxRestarts : 0
        }


        var exec_commands = [
            config.child_process.name,
            config.child_process.params
        ];

        
        var monitor = respawn(exec_commands, { maxRestarts : config.child_process.restart });
        
        console.log('launch child process: name:' + config.child_process.name + ", param:" +config.child_process.params + ", restart:" + config.child_process.restart);
 //       var spawn = require('child_process').spawn;
 //       var child_process = spawn(config.child_process.name, config.child_process.params);


        monitor.start();
        
        monitor.on('stdout', function(data){

            self.emit('stdout_data', data); 
        });

        monitor.on('stderr', function(data){

            self.emit('stderr_data', data); 
        });

        monitor.on('exit', function(code){


            self.emit('child_close', code);
            console.log('child process exited with code ' + code);
  
        });
        
    }
}

util.inherits(HumixModule, EventEmitter);

HumixModule.prototype.event = function(name,value) {

    var prefix = 'humix.sense.'+config.module_name+".event";

    var topic = prefix + "." + name;

    console.log("publish event with topic :" + topic);
    nats.publish(topic, value);
};


module.exports = HumixModule;
